/* FILE: driver.h                 -*-Mode: c++-*-
 *
 * Abstract driver class; fills states and steps evolvers.
 * Also registers the command "Oxs_Run" with the Tcl intepreter.
 *
 */

#ifndef _OXS_DRIVER
#define _OXS_DRIVER

#include "ext.h"
#include "simstate.h"
#include "key.h"
#include "evolver.h"	// Needed by MSVC++ 5

/* End includes */

class Oxs_Evolver; // Forward reference

class Oxs_Driver: public Oxs_Ext {
private:
  Oxs_ConstKey<Oxs_SimState> current_state;
  Oxs_Key<Oxs_Evolver> evolver_key;

protected:
  Oxs_Driver(const char* name,        // Child instance id
             Oxs_Director* newdtr,    // App director
             Tcl_Interp* safe_interp, // Safe interpreter
	     const char* argstr);     // Args; should include
                          /// an "evolver <evolver-name>" pair.

  BOOL Init_Driver();  // All children of Oxs_Driver *must* call
  /// this function in their Init() routines.  The main purpose
  /// of this function is to initialize the current state.

public:

  virtual ~Oxs_Driver();

  virtual Oxs_ConstKey<Oxs_SimState> GetInitialState() const =0;

  const Oxs_SimState* GetCurrentState() {
    return current_state.GetPtr();
  }

  int Run(string& result, int argc, char** argv,
	  Tcl_Interp* interp); // Called from director.

  // Generic interface.
  virtual BOOL Done(const Oxs_SimState& new_state, string& result) const =0;
  virtual BOOL StageDone(const Oxs_SimState& new_state,
			 string& result) const =0;
  virtual void NewState(const Oxs_SimState& new_state, string& result) const =0;
  virtual void FillStatePartA(const Oxs_SimState& old_state,
			      Oxs_SimState& new_state) const =0;
  virtual void FillStatePartB(Oxs_SimState& work_state) const =0;
  /// FillStatePartA is called to fill in a new blank state, using
  ///   an old state as a template.
  /// FillStatePartB is called by the evolver to complete a state
  ///   after evolver modifications.
};

#endif // _OXS_DRIVER


