/* FILE: simstate.h                 -*-Mode: c++-*-
 *
 * Simulation state class.
 *
 */

#ifndef _OXS_SIMSTATE
#define _OXS_SIMSTATE

#include "oc.h"
#include "lock.h"
#include "mesh.h"
#include "meshvalue.h"
#include "threevector.h"

/* End includes */

class Oxs_SimState : public Oxs_Lock {
  // Perhaps the inheritance from Oxs_Lock should be private, but then
  // exposing the Oxs_Lock member functions is awkward.
  // Note: To maintain a clean dependency hierarchy and avoid cyclic
  //  dependencies, Oxs_SimState objects should not contain any keys,
  //  because Oxs_SimState objects are held by Oxs_Ext objects which may
  //  have locks places on them by other Oxs_Ext objects.  Therefore,
  //  Oxs_SimState objects are placed at the root of the dependency
  //  hierarchy, and are deleted by Oxs_Director only after all Oxs_Ext
  //  objects are destroyed.  It is therefore the responsibility of each
  //  object holding a lock on an Oxs_SimState object to place locks on
  //  or otherwise guarantee the correctness of all pointers held within
  //  the Oxs_SimState object.
public:
  Oxs_SimState();
  ~Oxs_SimState();

  // The following elements should perhaps be wrapped up in a
  // STL map array, and indexed by name.
  UINT4m stage_number;     // Stage index
  UINT4m stage_iteration_count; // Number of iterations for this stage
  UINT4m iteration_count;  // Total number of distinct spin configurations
  REAL8m last_timestep; // Seconds
  REAL8m elapsed_time;  // Seconds

  // It is the responsibility of any object using this Oxs_SimState
  // object to insure the validity of these pointers.
  const Oxs_MeshValue<REAL8m>* Ms;
  const Oxs_MeshValue<REAL8m>* Ms_inverse; // 1/Ms
  const Oxs_Mesh* mesh;

  Oxs_MeshValue<ThreeVector> spin;
};

#endif // _OXS_SIMSTATE


